#! /usr/bin/env python3
# encoding: utf-8

# --------------------------------------------------

import os

# --------------------------------------------------

def boolean(arg):
	"""
	Converts
	- "True", "true" or "1" ⇒ True
	- "False", "false" or "0" ⇒ False
	
	@while converting argument to bool
	"""
	
	if arg in (True, "True", "true", "1"):
		return True
	elif arg in (False, "False", "false", "0"):
		return False
	else:
		raise ValueError(arg)

def booleans(*args):
	"""
	Converts each
	- "True", "true" or "1" ⇒ True
	- "False", "false" or "0" ⇒ False
	
	@while converting arguments to bools
	"""
	
	out = []
	
	for arg in args:
		out.append(boolean(arg))
	
	return out

def strings(*args):
	"""
	Converts each element to str.
	
	@while converting arguments to strings
	"""
	
	out = []
	
	for arg in args:
		out.append(str(arg))
	
	return out

def path(exists=False, permissions=None, validators=None):
	"""
	Returns a path callback that takes a path (str) and verifies if it:
	- exists iff `exists` is True
	- its permissions match those in `permissions` ("rwx" or any subset, e.g. "r--", "-w-", dashes are optional)
	- goes through each (function, message) pair in `validators`, in order
	
	A validator is a function that takes a str argument and returns True or False.
	If False is returned, the matching message is raised along with a RuntimeError.
	The message may contain %s to be replaced by the supplied path. Example:
	
	validators=[
		(os.path.isdir, "%s is not a directory"),
		(os.path.isabs, "%s is a directory, but is not an absolute path.")
	]
	
	@while generating a path callback
	"""
	
	def cb(arg):
		path = os.path.abspath(os.path.expanduser(arg))
		
		if exists and not os.path.exists(path):
			raise FileNotFoundError("%s does not exist" %(arg))
		
		if permissions:
			if "r" in permissions and not os.access(path, os.R_OK):
				raise PermissionError("%s is not readable" %(arg))
			if "w" in permissions and not os.access(path, os.W_OK):
				raise PermissionError("%s is not writable" %(arg))
			if "x" in permissions and not os.access(path, os.X_OK):
				raise PermissionError("%s is not executable" %(arg))
		
		if validators:
			if hasattr(validators, "__iter__"):
				src = validators
			else:
				src = [validators]
			
			for v in src:
				if hasattr(v, "__iter__"):
					fn, msg_tpl = v
				else:
					fn = v
					msg_tpl = "%%s failed validation by '%s'" %(fn.__name__)
				
				if not fn(path):
					msg = msg_tpl %(arg) if "%s" in msg_tpl else msg_tpl
					
					raise RuntimeError(msg)
		
		return path
	
	return cb

def integer(arg):
	"""
	Converts
	- "0x2a" ⇒ 42
	- "0o52" ⇒ 42
	- "42" ⇒ 42
	
	@while converting argument to int
	"""
	
	if type(arg) == int:
		return arg
	
	if len(arg) > 2:
		if arg[1] == "x":
			return int(arg, 16)
		elif arg[1] == "o":
			return int(arg, 8)
	
	return int(arg)

def integers(*args):
	"""
	@while converting arguments to ints
	"""
	
	out = []
	
	for arg in args:
		out.append(integer(arg))
	
	return out
